///////////////////////////////////////////////////////////////////////////////
//
//      TargaImage.cpp                          Author:     Stephen Chenney
//                                              Modified:   Eric McDaniel
//                                              Date:       Spring 2002
//
//      Implementation of TargaImage methods.  You must implement the image
//  modification functions.
//
///////////////////////////////////////////////////////////////////////////////

#include "Globals.h"
#include "TargaImage.h"
#include "Lai_struct.h"

#include <libtarga.h>
#include <stdlib.h>
#include <assert.h>
#include <memory.h>
#include <math.h>
#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

// constants
const int           RED             = 0;                // red channel
const int           GREEN           = 1;                // green channel
const int           BLUE            = 2;                // blue channel
const unsigned char BACKGROUND[3]   = { 0, 0, 0 };      // backgraound color

// Global variable
unsigned char median_index[752];
int           median_index_size;


///////////////////////////////////////////////////////////////////////////////
//
//      Constructor.  Initialize member variables.
//
///////////////////////////////////////////////////////////////////////////////
TargaImage::TargaImage() : width(0), height(0), image_size(0), data(NULL),
						   rgb_data(NULL), gray_data(NULL)
{}// TargaImage


///////////////////////////////////////////////////////////////////////////////
//
//      Constructor.  Initialize member variables to values given.
//
///////////////////////////////////////////////////////////////////////////////
TargaImage::TargaImage(int w, int h, unsigned char *d)
{
    int i;

    width = w;
    height = h;
    data = new unsigned char[width * height * 4];

	gray_data = NULL;
	rgb_data = NULL;
	image_size = width * height;

    for (i = 0; i < width * height * 4; i++)
	    data[i] = d[i];
}// TargaImage


///////////////////////////////////////////////////////////////////////////////
//
//      Destructor.  Free image memory.
//
///////////////////////////////////////////////////////////////////////////////
TargaImage::~TargaImage()
{
    if (data)
        delete[] data;
	if (rgb_data)
		delete[] rgb_data;
	if (gray_data)
		delete[] gray_data;
}// ~TargaImage


///////////////////////////////////////////////////////////////////////////////
//
//      Converts an image to RGB form, and returns the rgb pixel data - 24 
//  bits per pixel. The returned space should be deleted when no longer 
//  required.
//
///////////////////////////////////////////////////////////////////////////////
unsigned char* TargaImage::To_RGB(void)
{
    unsigned char   *rgb = new unsigned char[width * height * 3];
    int		    i, j;

    if (! data)
	    return NULL;

    // Divide out the alpha
    for (i = 0 ; i < height ; i++)
    {
	    int in_offset = i * width * 4;
	    int out_offset = i * width * 3;

	    for (j = 0 ; j < width ; j++)
        {
	        RGBA_To_RGB(data + (in_offset + j*4), rgb + (out_offset + j*3));
	    }
    }

    return rgb;
}// TargaImage


///////////////////////////////////////////////////////////////////////////////
//
//      Save the image to a targa file. Returns 1 on success, 0 on failure.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Save_Image(char *filename)
{
    TargaImage	*out_image = Reverse_Rows();

    if (! out_image)
	    return false;

    if (!tga_write_raw(filename, width, height, out_image->data, TGA_TRUECOLOR_32))
    {
	    cout << "TGA Save Error: %s\n", tga_error_string(tga_get_last_error());
	    return false;
    }

    delete out_image;

    return true;
}// Save_Image


///////////////////////////////////////////////////////////////////////////////
//
//      Load a targa image from a file.  Return a new TargaImage object which 
//  must be deleted by caller.  Return NULL on failure.
//
///////////////////////////////////////////////////////////////////////////////
TargaImage* TargaImage::Load_Image(char *filename)
{
    unsigned char   *temp_data;
    TargaImage	    *temp_image;
    TargaImage	    *result;
    int		        width, height;

    if (!filename)
    {
        cout << "No filename given." << endl;
        return NULL;
    }// if

    temp_data = (unsigned char*)tga_load(filename, &width, &height, TGA_TRUECOLOR_32);
    if (!temp_data)
    {
        cout << "TGA Error: %s\n", tga_error_string(tga_get_last_error());
	    width = height = 0;
	    return NULL;
    }
    temp_image = new TargaImage(width, height, temp_data);
    free(temp_data);

    result = temp_image->Reverse_Rows();

    delete temp_image;

    return result;
}// Load_Image


///////////////////////////////////////////////////////////////////////////////
//
//      Convert image to grayscale.  Red, green, and blue channels should all 
//  contain grayscale value.  Alpha channel shoould be left unchanged.  Return
//  success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::To_Grayscale()
{
	// Temporary variable
	int i, j;
	//Variable for calculating the starting point of each image
	int offset;
	// The temporary memory space for rgb
	double rgb_temp[3];
	
	// Create the memory space for putting the gray image
	gray_data = new double[image_size];

	for(i = 0; i < image_size ; i ++){
		offset = i * 4;

		//get the rgb data
		if(data[offset + 3]){
			// test using the To_RGB;
			// R = r * alpha * 255; Alpha = 255 * alpha; R / Alpha = r
			for(j = 0; j < 3; j++)
				rgb_temp[j] = (double)data[offset + j] / data[offset + 3];
		}

		//Alpha = 0 means the pixel color is black
		else{
			for(j = 0; j < 3; j++)
				rgb_temp[j] = 0.0;
		}

		// Transform to grayscale
		gray_data[i] =  rgb_temp[0] * 0.299 + 
						rgb_temp[1] * 0.587 +
						rgb_temp[2] * 0.114;

		// Tranform gray data to R = r * (alpha * 255) = r * Alpha
		for(j = 0; j < 3; j++)
			data[offset + j] = gray_data[i] * data[offset + 3];
	}//end of for i

	return true;
}// To_Grayscale


///////////////////////////////////////////////////////////////////////////////
//
//  Convert the image to an 8 bit image using uniform quantization.  Return 
//  success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Quant_Uniform()
{
	// The use 3-bit R, 3-bit G, and 2-bit B
	const int R_Q_BIT = 3;
	const int G_Q_BIT = 3;
	const int B_Q_BIT = 2;
	
	// Quantum scaling factor
	int	      scale[3];
	int		  i, j;
	int       in_offset, out_offset;
	int       temp = 0;

	unsigned  char *RGB_data;

	// Getting the scaling factor
	//scale[0] = pow(2, 8) / pow(2, R_Q_BIT);
	//scale[1] = pow(2, 8) / pow(2, G_Q_BIT);
	//scale[2] = pow(2, 8) / pow(2, B_Q_BIT);
	

	scale[0] = 8 - R_Q_BIT;
	scale[1] = 8 - G_Q_BIT;
	scale[2] = 8 - B_Q_BIT;

	// Transform RGBA to RGB 
	RGB_data = To_RGB();

	// Quantumize each pixel
	for(i = 0; i < image_size; i++){
		in_offset = 3 * i;
		out_offset = 4 * i;
		for(j = 0; j < 3; j++){
			// Directly truncate the result
			// it use directly truncate not round off

			RGB_data[in_offset + j] = RGB_data[in_offset + j] >> scale[j];
			RGB_data[in_offset + j] = RGB_data[in_offset + j] << scale[j];

			// R = 255 * r * alpha = r * (alpha* 255) = r * Alpha 
			data[out_offset + j] = (double)RGB_data[in_offset + j] / 255.0
									* data[out_offset + 3];
		} 
	}
	delete [] RGB_data;
	return true;
}// Quant_Uniform


///////////////////////////////////////////////////////////////////////////////
//
//      Convert the image to an 8 bit image using populosity quantization.  
//  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Quant_Populosity()
{
	// Temporary variable for quantizing rgb index
	unsigned char rgb[3];
	// To hold the rgb of the entire image
	unsigned char *rgb_color;
	// The index for the quantized color space
	int           index;
	// The starting point
	int           offset;
	// The output image starting point
	int           out_offset;
	// temporary variable
	int           i, j;
	// The histogram of each image
	color_ele	  *histogram[32768];
	// The creating looking up table for this image
	unsigned char index_color[98304];
	// The minimun distance sqare
	double        distance_min2;
	// The distance square to the reference 256 color
	double        distance2;
	// diff
	double        d_r, d_g, d_b;

	rgb_color = To_RGB();

	
	for(i = 0; i < 32768; i++){
		histogram[i] = new color_ele;
		histogram[i]->r = i % 32;
		histogram[i]->g = (i % 1024) / 32 ;
		histogram[i]->b = i / 1024;
		histogram[i]->occurence = 0;
	}

	// Do the statistical histogram
	for(i = 0; i < image_size; i++){
		offset = 3 * i;
		for(j = 0; j < 3; j++)
			rgb[j] = rgb_color[offset + j] >> 3;
		index = (int)rgb[0] + (int)rgb[1] *32 + (int)rgb[2] * 1024;
		histogram[index]->occurence++;
	}

	// Sorting the occurence
	qsort(histogram, 32768, sizeof(color_ele *), compare);

	// put the first 256 color into the index color array
	for(i = 32512; i < 32768; i++){
		offset =   (int) histogram[i]->r 
				 + (int) histogram[i]->g * 32 
				 + (int) histogram[i]->b * 1024;
		offset *= 3;
		index_color[offset]     = histogram[i]->r << 3;
		index_color[offset + 1] = histogram[i]->g << 3;
		index_color[offset + 2] = histogram[i]->b << 3;
	}

	// Find out the minimum distance to the 256 color array and then
	// put it into the corresponding index array
	for(i = 0; i < 32512; i++){
		distance_min2 = 10000.0;
		index         = 0;
		for(j = 32512; j < 32768; j++){

			// Calculate hte distance to each index color
			d_r = histogram[i]->r - histogram[j]->r;
			d_g = histogram[i]->g - histogram[j]->g;
			d_b = histogram[i]->b - histogram[j]->b;
			distance2 = d_r * d_r + d_g * d_g + d_b * d_b;

			//Find out the minimum color and index of the color
			if(distance2 < distance_min2){
				distance_min2 = distance2;
				index = j;
			}// end of if
		}// end of for j

		// Calculate the index in the index color array
		offset =   (int) histogram[i]->r + (int) histogram[i]->g * 32 
				 + (int) histogram[i]->b * 1024;
		// The starting point of the index array;
		offset *= 3;

		// put each minimum distance 256 color into the index color array
		index_color[offset]     = histogram[index]->r << 3;
		index_color[offset + 1] = histogram[index]->g << 3;
		index_color[offset + 2] = histogram[index]->b << 3;
	}

	for(i = 0; i < image_size; i++){
		offset = 3 * i;
		out_offset = 4 * i;
		for(j = 0; j < 3; j++)
			rgb[j] = rgb_color[offset + j] >> 3;
		
		// Find out the index in the index color table
		index = (int)rgb[0] + (int)rgb[1] *32 + (int)rgb[2] * 1024;
		index *= 3;
		for(j = 0; j < 3; j++){
			rgb_color[offset + j] = index_color[index + j];
			data[out_offset + j] = (double) index_color[index + j]
									* data[out_offset + 3] / 255.0;
		}
	}
		
	for(i = 0; i < 32768; i++)
		delete histogram[i];

	delete [] rgb_color;
	return true;
}// Quant_Populosity

///////////////////////////////////////////////////////////////////////////////
//
//      Determine the median of each sub unit and create the decision tree
//
///////////////////////////////////////////////////////////////////////////////

d_tree *TargaImage::median(int n, unsigned char *image_data, int size)
{
	// store the maximum and minimum value image data
    int rgb_max[3], rgb_min[3];
	// range of the rgb
    int range[3];
	// Store the statistical histogram data
    int R_histogram[256], G_histogram[256], B_histogram[256];
	// Temporary data
    int i, j, k, l;
	int temp;
	// Starting point
    int offset;
	// Starting point of smaller, and starting point of greater than median
    int left_offset, right_offset;
	// Pointer of the smaller and greater array
    unsigned char *left, *right; 

	// num is the total number below median, num1 is for the second layer, num2 is for the third
    int num = 0, num1, num2;
	
	// node for the decision tree
	d_tree *node;
	
	//Create the tree node
	node = new d_tree;
	
	node->left = NULL;
	node->right = NULL;

	// only operate 0 ~ 8 layer to get 256 sub-cubes
    if(n > 7)
      return NULL;

	// Initialize the R, G, B histogram
    for(i = 0; i < 256 ; i ++){
      R_histogram[i] = 0;
      G_histogram[i] = 0;
      B_histogram[i] = 0;
    }
    
	// calculate the histogram of each image
    for(i = 0; i < size; i++){
      offset = i * 3;
      R_histogram[image_data[offset]]++;
      G_histogram[image_data[offset + 1]]++;
      B_histogram[image_data[offset + 2]]++;
    }

	// Find the maximum and minimum of the image in R
    for(j = 0; j < 256; j++)
		if(R_histogram[j])
			break;
	if(j == 256)
		rgb_min[0] = 255;
	else
		rgb_min[0] = j;

    for(j = 255; j >= 0 ; j--)
		if(R_histogram[j])
			break;
	if(j < 0)
		rgb_max[0] = 0;
	else
		rgb_max[0] = j;

	// Find the maximum and minimum of the image in G
    for(j = 0; j < 256; j++)
		if(G_histogram[j])
			break;
	if(j == 256)
		rgb_min[1] = 255;
	else
		rgb_min[1] = j;

    for(j = 255; j >= 0 ; j--)
		if(G_histogram[j])
			break;
	if(j < 0)
		rgb_max[1] = 0;
	else
		rgb_max[1] = j;

	// Find the maximum and minimum of the image in B
    for(j = 0; j < 256; j++)
		if(B_histogram[j])
			break;
	if(j == 256)
		rgb_min[2] = 255;
	else
		rgb_min[2] = j;

    for(j = 255; j >= 0 ; j--)
		if(B_histogram[j])
			break;
	if(j < 0)
		rgb_max[2] = 0;
	else
		rgb_max[2] = j;

	// Print constant
	int	m = 1;

	// Find out the range
    for(j = 0; j < 3; j++)
      range[j] = rgb_max[j] - rgb_min[j];

	// Find the median of the maximum range
	// First layer of RED
    if(range[0] >= range[1] && range[0] >= range[2]){
		//R's range is maximum and find out the median in R
		for(j = 0; j < 256; j++){
			num += R_histogram[j];
			if(num >= size/2)
				break;
		}
		// The first cut is on RED but the total number be sure on the left is num - the row 
		// that contain the median
		// set up the first cut flag 0 for RED, 1 for GREEN, 2 for BLUE
		node->cut0 = 0;
		// Put the color in the tree node
		node->median[0] = j;
		// The number is sure to be the left of the median
		num -= R_histogram[j];
		// The number for second run
		num1 = R_histogram[j];

		// Reinitialize and we hope to find out the median in GB plane
		for(j = 0; j < 256; j++){
			G_histogram[j] = 0;
			B_histogram[j] = 0;
		}

		// Calculate the statistics in GB in color in median-RED
	    for(i = 0; i < size; i++){
			offset = i * 3;
			// calculate the histogram of each image in GB in median-RED
			if(image_data[offset] == node->median[0]){
				G_histogram[image_data[offset + 1]]++;
				B_histogram[image_data[offset + 2]]++;
			}
		}

		// Find the maximum and minimum of the image in G
		for(j = 0; j < 256; j++)
			if(G_histogram[j])
				break;
		if(j == 256)
			rgb_min[1] = 255;
		else
			rgb_min[1] = j;

		for(j = 255; j >= 0 ; j--)
			if(G_histogram[j])
				break;
		if(j < 0)
			rgb_max[1] = 0;
		else
			rgb_max[1] = j;

		// Find the maximum and minimum of the image in B
		for(j = 0; j < 256; j++)
			if(B_histogram[j])
				break;
		if(j == 256)
			rgb_min[2] = 255;
		else
			rgb_min[2] = j;

		for(j = 255; j >= 0 ; j--)
			if(B_histogram[j])
				break;
		if(j < 0)
			rgb_max[2] = 0;
		else
			rgb_max[2] = j;

		range[1] = rgb_max[1] - rgb_min[1];
		range[2] = rgb_max[2] - rgb_min[2];

		// Start the second layer median searching
		// We choose the maximum range in median RED
		// Which is GREEN 
		if(range[1] >= range[2]){
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += G_histogram[j];
				if(temp >= num1/2)
					break;
			}
			// Set the second cut flag to be GREEN
			node->cut1 = 1;
			// Set the second cut median color of GREEN
			node->median[1] = j;
			// The color sure to be the left of the Med-G is temp - line of the MED-G
			temp -= G_histogram[j];
			// The color sure to be the median-R, G is num + temp
			num += temp ;
			// The color we not sure is on the median-R, G plane
			num2 = G_histogram[j];

			for(j = 0; j < 256; j++)
				B_histogram[j] = 0;

		    for(i = 0; i < size; i++){
				offset = i * 3;
				// calculate the histogram of each image on the median-R, G plane
				if( image_data[offset]     == node->median[0] &&
					image_data[offset + 1] == node->median[1])
					B_histogram[image_data[offset + 2]]++;
			}
			temp = 0;
			for(j = 0; j< 256; j++){
				temp += B_histogram[j];
				if(temp >= num2/2)
					break;
			}
			// Set the number on the left
			num += temp;
			// Set the color of last element
			node->median[2] = j;
			
		}// end of if range[1] >= range[2]
		// Start the second layer median searching
		// We choose the maximum range in median RED
		// Which is BLUE 
		else{
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += B_histogram[j];
				if(temp >= num1/2)
					break;
			}
			// Set the second cutting point is BLUE
			node->cut1 = 2;
			// Set the second cutting color on blue element
			node->median[2] = j;
			// The number sure to be the left of BLUE on med-RED is temp - the line on median
			temp -= B_histogram[j];
			// The total number sure to be the left of RED-BLUE is num + temp
			num += temp ;
			// The number of node not sure
			num2 = B_histogram[j];

			for(j = 0; j < 256; j++)
				G_histogram[j] = 0;

		    for(i = 0; i < size; i++){
				offset = i * 3;
				// calculate the histogram of each image on med-R, B plane
				if(	image_data[offset]     == node->median[0] &&
					image_data[offset + 2] == node->median[2])
					G_histogram[image_data[offset + 1]]++;
			}
			temp = 0;
			for(j = 0; j< 256; j++){
				temp += G_histogram[j];
				if(temp >= num2/2)
					break;
			}

			num += temp;
			node->median[1] = j;
		}// end second layer which is range[1] >= range[0]
	}// if(range[0] >= range[1] && range[0] >= range[2])

	// Find the median of the maximum range which is GREEN
	else if(range[1] >= range[2] && range[1] > range[0]){
		//G's range is maximum and find out the median in G
		for(j = 0; j < 256; j++){
			num += G_histogram[j];
			if(num >= size/2)
				break;
		}
		// The first cut is GREEN 
		// Set the flag
		node->cut0 = 1;
		// Set the GREEN color
		node->median[1] = j;
		// The number sure in the left
		num -= G_histogram[j];
		// The number not sure which side
		num1 = G_histogram[j];

		// Reinitialize and we hope to find out the median in RB plane
		for(j = 0; j < 256; j++){
			R_histogram[j] = 0;
			B_histogram[j] = 0;
		}
		// Calculate the statistics histogram in RB in median-G
	    for(i = 0; i < size; i++){
			offset = i * 3;
			// calculate the histogram of each image in median-G
			if(image_data[offset + 1] == node->median[1]){
				R_histogram[image_data[offset]]++;
				B_histogram[image_data[offset + 2]]++;
			}
		}

		// Find the maximum and minimum of the image in R
		for(j = 0; j < 256; j++)
			if(R_histogram[j])
				break;
		if(j == 256)
			rgb_min[0] = 255;
		else
			rgb_min[0] = j;

		for(j = 255; j >= 0 ; j--)
			if(R_histogram[j])
				break;
		if(j < 0)
			rgb_max[0] = 0;
		else
			rgb_max[0] = j;

		// Find the maximum and minimum of the image in B
		for(j = 0; j < 256; j++)
			if(B_histogram[j])
				break;
		if(j == 256)
			rgb_min[2] = 255;
		else
			rgb_min[2] = j;

	    for(j = 255; j >= 0 ; j--)
			if(B_histogram[j])
				break;
		if(j < 0)
			rgb_max[2] = 0;
		else
			rgb_max[2] = j;

		range[0] = rgb_max[0] - rgb_min[0];
		range[2] = rgb_max[2] - rgb_min[2];

		// Start of the second layer
		// We choose the maximum range in med-G
		// Which is the R 
		if(range[0] >= range[2]){
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += R_histogram[j];
				if(temp >= num1/2)
					break;
			}

			// Set up the secod cut flag to be RED
			node->cut1 = 0;
			// Set the red color
			node->median[0] = j;
			// the number sure to be the left
			temp -= R_histogram[j];
			// The total number to be sure at G-R plane left
			num += temp ;
			// The number not sure
			num2 = R_histogram[j];

			for(j = 0; j < 256; j++)
				B_histogram[j] = 0;

		    for(i = 0; i < size; i++){
				offset = i * 3;
				// calculate the histogram of each image
				if(	image_data[offset]     == node->median[0] &&
					image_data[offset + 1] == node->median[1])
					B_histogram[image_data[offset + 2]]++;
			}	
			
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += B_histogram[j];
				if(temp >= num2/2)
					break;
			}

			num += temp;
			node->median[2] = j;
			
		}// end of if range[0] >= range[2]

		// We choose the maximum range in Med-G
		// Which is B component
		else{
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += B_histogram[j];
				if(temp >= num1/2)
					break;
			}

			// Set the cut flag to be BLUE
			node->cut1 = 2;
			// Set the cut color
			node->median[2] = j;
			// The number sure to be the left on med-G
			temp -= B_histogram[j];
			// THe number sure to be the left on med-G, B
			num += temp ;
			// The number not sure
			num2 = B_histogram[j];

			for(j = 0; j < 256; j++)
				R_histogram[j] = 0;

		    for(i = 0; i < size; i++){
				offset = i * 3;
				// calculate the histogram of each image
				if(	image_data[offset + 1] == node->median[1] &&
					image_data[offset + 2] == node->median[2])
					R_histogram[image_data[offset + 1]]++;
			}//end of for i
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += R_histogram[j];
				if(temp >= num2/2)
					break;
			}
			num += temp;
			node->median[0] = j;
		}// end second layer // end of if range[0] >= range[2]
	}// end first layer else if
	// Find the median of the maximum range
	// First layer of blue
    else {
		//B's range is maximum and find out the median in B
		for(j = 0; j< 256; j++){
			num += B_histogram[j];
			if(num >= size/2){
				break;
			}
		}
		// The first cut is Blue 
		// Set the cut flag to be 2
		node->cut0 = 2;
		// Set the cut color to be 2
		node->median[2] = j;
		// The number sure to be the left
		num -= B_histogram[j];
		// The number not sure
		num1 = B_histogram[j];

		// Reinitialize and we hope to find out the median-Blue
		for(j = 0; j < 256; j++){
			R_histogram[j] = 0;
			G_histogram[j] = 0;
		}
		// Calculate the statistics in GB in color j
	    for(i = 0; i < size; i++){
			offset = i * 3;
			// calculate the histogram of each image in R
			if(image_data[offset + 2] == node->median[2]){
				R_histogram[image_data[offset]]++;
				G_histogram[image_data[offset + 1]]++;
			}
		}

		// Find the maximum and minimum of the image in R
		for(j = 0; j < 256; j++)
			if(R_histogram[j])
				break;
		if(j == 256)
			rgb_min[0] = 255;
		else
			rgb_min[0] = j;

		for(j = 255; j >= 0 ; j--)
			if(R_histogram[j])
				break;
		if(j < 0)
			rgb_max[0] = 0;
		else
			rgb_max[0] = j;

		// Find the maximum and minimum of the image in G
		for(j = 0; j < 256; j++)
			if(G_histogram[j])
				break;
		if(j == 256)
			rgb_min[1] = 255;
		else
			rgb_min[1] = j;

		for(j = 255; j >= 0 ; j--)
			if(G_histogram[j])
				break;
		if(j < 0)
			rgb_max[1] = 0;
		else
			rgb_max[1] = j;

		range[0] = rgb_max[0] - rgb_min[0];
		range[1] = rgb_max[1] - rgb_min[1];

		// Start of the second layer
		// We choose the maximum range in med-B
		// Which is GREEN 
		if(range[1] > range[0]){
			temp = 0;
			for(j = 0; j< 256; j++){
				temp += G_histogram[j];
				if(temp >= num1/2)
					break;
			}
			// Set up the cut flag
			node->cut1 = 1;
			// Set up the cut color to be green
			node->median[1] = j;
			// The number sure to be the left
			temp -= G_histogram[j];
			// The total number to be sure at the left
			num += temp ;
			// The number not sure
			num2 = G_histogram[j];

			for(j = 0; j < 256; j++)
				R_histogram[j] = 0;

		    for(i = 0; i < size; i++){
				offset = i * 3;
				// calculate the histogram of each image
				if(	image_data[offset + 1] == node->median[1] &&
					image_data[offset + 2] == node->median[2])
					R_histogram[image_data[offset]]++;
			}// end of for i
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += R_histogram[j];
				if(temp >= num2/2)
					break;
			}

			num += temp;
			node->median[0] = j;
		}// end of if range[1] > range[0]
		// We choose the maximum range in med-B
		// Which is RED
		else{
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += R_histogram[j];
				if(temp >= num1/2)
					break;
			}
			// Set up the cut flag
			node->cut1 = 0;
			// set up the cut color
			node->median[0] = j;
			// The number sure to be left
			temp -= R_histogram[j];
			// The number sure to be left
			num += temp ;
			// The number not sure
			num2 = R_histogram[j];

			for(j = 0; j < 256; j++)
				G_histogram[j] = 0;

		    for(i = 0; i < size; i++){
				offset = i * 3;
				// Find out the maximum and minimum of rgb
				// calculate the histogram of each image
				if(	image_data[offset]     == node->median[0] &&
					image_data[offset + 2] == node->median[2])
					G_histogram[image_data[offset + 1]]++;
			}// end of for i
			temp = 0;
			for(j = 0; j < 256; j++){
				temp += G_histogram[j];
				if(temp >= num2/2)
					break;
			}

			num += temp;
			node->median[1] = j;
		}// end second layer which is if(range[1] > range[0])
	}// end first layer which is if(range[0] > = range[1] &&..)

	left  = new unsigned char[size * 3];
	right = new unsigned char[size * 3];
	
	// index of the left and right size
	k = 0;
	l = 0;

	// Create the left and right image
	for(i = 0; i < size; i++){
		offset = i * 3;
		// Start of first layer at red
		if(node->cut0 == 0){
			if(image_data[offset] < node->median[0]){
				left_offset = k * 3;
				for(j = 0; j < 3; j++)
					left[left_offset + j] = image_data[offset + j];
				k++;
			}
			else if(image_data[offset] > node->median[0]){
				right_offset = l * 3;
				for(j = 0; j < 3; j++)
					right[right_offset + j] = image_data[offset + j];
				l++;
			}
			//start of second layer at GREEN or BLUE
			else{
				if(node->cut1 == 1){
					// start at GREEN
					if(image_data[offset + 1] < node->median[1]){
						left_offset = k * 3;
						for(j = 0; j < 3; j++)
							left[left_offset + j] = image_data[offset + j];
						k++;
					}
					else if(image_data[offset + 1] > node->median[1]){
						right_offset = l * 3;
						for(j = 0; j < 3; j++)
							right[right_offset + j] = image_data[offset + j];
						l++;
					}
					// Start of the third layer at BLUE
					else{
						if(image_data[offset + 2] < node->median[2]){
							left_offset = k * 3;
							for(j = 0; j < 3; j++)
								left[left_offset + j] = image_data[offset + j];
							k++;
						}
						else if(image_data[offset + 2] > node->median[2]){
							right_offset = l * 3;
							for(j = 0; j < 3; j++)
								right[right_offset + j] = image_data[offset + j];
							l++;
						}// end of if <=
					}//end of if(image_data[offset + 1] < node->median[1])
				}// end of if(node->cut1 == 1)
				else{
					// Start at blue
					if(image_data[offset + 2] < node->median[2]){
						left_offset = k * 3;
						for(j = 0; j < 3; j++)
							left[left_offset + j] = image_data[offset + j];
						
						k++;
						
					}
					else if(image_data[offset + 2] > node->median[2]){
						right_offset = l * 3;
						for(j = 0; j < 3; j++)
							right[right_offset + j] = image_data[offset + j];
						l++;
					}
					// Start at green
					else{
						if(image_data[offset + 1] < node->median[1]){
							left_offset = k * 3;
							for(j = 0; j < 3; j++)
								left[left_offset + j] = image_data[offset + j];
							k++;
						}
						else if(image_data[offset + 1] > node->median[1]){
							right_offset = l * 3;
							for(j = 0; j < 3; j++)
								right[right_offset + j] = image_data[offset + j];
							l++;
						}// end of if <=
					}//end of if(image_data[offset + 2] < node->median[2])
				} // end of if(node->cut1 == 1)
			}// end of if(image_data[offset] < node->median[0])
		}//if(node->cut0 == 0)
		else if(node->cut0 == 1){
			// Start with GREEN
			if(image_data[offset + 1] < node->median[1]){
				left_offset = k * 3;
				for(j = 0; j < 3; j++)
					left[left_offset + j] = image_data[offset + j];
				k++;
			}
			else if(image_data[offset + 1] > node->median[1]){
				right_offset = l * 3;
				for(j = 0; j < 3; j++)
					right[right_offset + j] = image_data[offset + j];
				l++;
			}
			//start of second layer
			else{
				// Start with RED
				if(node->cut1 == 0){
					if(image_data[offset] < node->median[0]){
						left_offset = k * 3;
						for(j = 0; j < 3; j++)
							left[left_offset + j] = image_data[offset + j];
						k++;
					}
					else if(image_data[offset] > node->median[0]){
						right_offset = l * 3;
						for(j = 0; j < 3; j++)
							right[right_offset + j] = image_data[offset + j];
						l++;
					}
					else{
						// Start of the third layer at BLUE
						if(image_data[offset + 2] < node->median[2]){
							left_offset = k * 3;
							for(j = 0; j < 3; j++)
								left[left_offset + j] = image_data[offset + j];
							k++;
						}
						else if(image_data[offset + 2] > node->median[2]){
							right_offset = l * 3;
							for(j = 0; j < 3; j++)
								right[right_offset + j] = image_data[offset + j];
							l++;
						}// end of if <=
					}//end of if(image_data[offset] < node->median[0])
				}// end of if(node->cut1 == 1)
				else{
					// Start with blue
					if(image_data[offset + 2] < node->median[2]){
						left_offset = k * 3;
						for(j = 0; j < 3; j++)
							left[left_offset + j] = image_data[offset + j];
						k++;
					}
					else if(image_data[offset + 2] > node->median[2]){
						right_offset = l * 3;
						for(j = 0; j < 3; j++)
							right[right_offset + j] = image_data[offset + j];
						l++;
					}
					else{
						if(image_data[offset] < node->median[0]){
							left_offset = k * 3;
							for(j = 0; j < 3; j++)
								left[left_offset + j] = image_data[offset + j];
							k++;
						}
						else if(image_data[offset] > node->median[0]){
							right_offset = l * 3;
							for(j = 0; j < 3; j++)
								right[right_offset + j] = image_data[offset + j];
							l++;
						}// end of if <=
					}//end of if(image_data[offset + 2] < node->median[2])
				}// end of if(node->cut1 == 1)
			}// end of (image_data[offset + 1] < node->median[1])
		}// end of else if(node->cut0 == 1)
		else{
			//Start with Blue
			if(image_data[offset + 2] < node->median[2]){
				left_offset = k * 3;
				for(j = 0; j < 3; j++)
					left[left_offset + j] = image_data[offset + j];
				k++;
			}
			else if(image_data[offset + 2] > node->median[2]){
				right_offset = l * 3;
				for(j = 0; j < 3; j++)
					right[right_offset + j] = image_data[offset + j];
				l++;
			}
			else{
				// Start with Red
				if(node->cut1 == 0){
					if(image_data[offset] < node->median[0]){
						left_offset = k * 3;
						for(j = 0; j < 3; j++)
							left[left_offset + j] = image_data[offset + j];
						k++;
					}
					else if(image_data[offset] > node->median[0]){
						right_offset = l * 3;
						for(j = 0; j < 3; j++)
							right[right_offset + j] = image_data[offset + j];
						l++;
					}
					else{
						// Start with G
						if(image_data[offset + 1] < node->median[1]){
							left_offset = k * 3;
							for(j = 0; j < 3; j++)
								left[left_offset + j] = image_data[offset + j];
							k++;
						}
						else if(image_data[offset + 1] > node->median[1]){
							right_offset = l * 3;
							for(j = 0; j < 3; j++)
								right[right_offset + j] = image_data[offset + j];
							l++;
						}// end of if <=
					}//end of if(image_data[offset] < node->median[0])
				}// end of if(node->cut1 == 1)
				else{
					//Start with G
					if(image_data[offset + 1] < node->median[1]){
						left_offset = k * 3;
						for(j = 0; j < 3; j++)
							left[left_offset + j] = image_data[offset + j];
						k++;
					}
					else if(image_data[offset + 1] > node->median[1]){
						right_offset = l * 3;
						for(j = 0; j < 3; j++)
							right[right_offset + j] = image_data[offset + j];
						l++;
					}
					else{
						if(image_data[offset] < node->median[0]){
							left_offset = k * 3;
							for(j = 0; j < 3; j++)
								left[left_offset + j] = image_data[offset + j];
							k++;
						}
						else if(image_data[offset] > node->median[0]){
							right_offset = l * 3;
							for(j = 0; j < 3; j++)
								right[right_offset + j] = image_data[offset + j];
							l++;
						}// end of if <=
					}//end of if(image_data[offset + 1] < node->median[1])
				}// end of if(node->cut1 == 0)
			}// end of (image_data[offset + 2] < node->median[2])
		}// end of else if(node->cut0 == 1)
	}// end of for
		
	
	offset = 3 * median_index_size;
	for(j = 0; j < 3; j ++)
		median_index[offset + j] = node->median[j];

	median_index_size++;

	node->left  = median(n + 1, left, k);
    node->right = median(n + 1, right, l);

    delete [] left;
    delete [] right;
	return node;
}

///////////////////////////////////////////////////////////////////////////////
//
//      destruct the tree
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::delete_tree(d_tree *node)
{	
	if(!node)
		return;
	if(node->left)
		delete_tree(node->left);
	if(node->right)
		delete_tree(node->right);
	delete node;
	return;

}


///////////////////////////////////////////////////////////////////////////////
//
//      Use median cut quantization to convert image to an 8 bit color image.  
//  Return success of opertion.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Quant_Median()
{
	// root of the decision tree
	d_tree *root;

	// Temporary variable
	int i, j;
	// starting point
	int data_offset, index_offset, rgb_offset;
	// Variable for image
	unsigned char *image_data;
	// The variable to store minimum distance and current distance
	double        distance_min2, distance2;
	// The difference between centroid and current color
	double        d_r, d_g, d_b;
	// Indicate the minimum distance color
	int           index;

	// Transform the image data to RGB
	image_data = To_RGB();

	// Reset the index to be one
	median_index_size = 0;
	// Create the index median color table
	root = median(0, image_data, image_size);

	// Find out the minimum distance color and use it as color
	for(i = 0; i < image_size; i++){

		// Calculate the starting point of the pixel
		rgb_offset    = 3 * i;
		// Set the distance to be incredible long
		distance_min2 = 10000.0;
		index         = 0;

		for(j = 0; j < median_index_size; j++){

			// Calculate the starting point of the index color
			index_offset = 3 * j;
			// Calculate hte distance to each index color
			d_r = (double)median_index[index_offset]     - image_data[rgb_offset];
			d_g = (double)median_index[index_offset + 1] - image_data[rgb_offset + 1];
			d_b = (double)median_index[index_offset + 2] - image_data[rgb_offset + 2];

			distance2 = d_r * d_r + d_g * d_g + d_b * d_b;

			//Find out the minimum color and index of the color
			if(distance2 < distance_min2){
				distance_min2 = distance2;
				index = j;
			}// end of if
		}// end of for j

		data_offset  = 4 * i;
		index_offset = 3 * index;

		for(j = 0; j < 3; j++)
			data[data_offset + j] = (double) median_index[index_offset + j]
									* data[data_offset + 3] / 255.0;
	}

	delete [] image_data;
	delete_tree(root);
	return true;
}// Quant_Median


///////////////////////////////////////////////////////////////////////////////
//
//      Dither the image using a threshold of 1/2.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Dither_Threshold()
{
	// Temporary variable
	int i, j;
	// The starting point
	int offset;
	// Transform the image to gray scale
	To_Grayscale();

	for(i = 0; i < image_size; i++){
		// Check the threshold 0.5
		if(gray_data[i] < 0.5)
			gray_data[i] = 0.0;
		else
			gray_data[i] = 1.0;
		
		// Check the Alpha component
		offset = 4 * i;
		if(data[offset + 3] < 128){
			data[offset + 3] = 0;
			for(j = 0; j < 3; j++)
				data[offset + j] = 0;
		}

		else{
			data[offset + 3] = 255;
			for(j = 0; j < 3; j++)
				data[offset + j] = gray_data[i] * 255.0;
		}
	}

    return true;
}// Dither_Threshold

///////////////////////////////////////////////////////////////////////////////
//
//      Dither the image while conserving the average brightness.  Return 
//  success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Dither_Bright()
{
	// Temporary variable;
	int    i, j;
	// The image starting point
	int    offset;
	// Statistical result of the image 
	int    histogram[256];
	// the average intensity of the image
	double intensity_ave = 0;
	double gray_intensity = 0;
	// The total black number of pixels should be in this image
	int    dark_number;
	// The threshold for the image
	double threshold = 0;


	// Transform the image to gray scale
	To_Grayscale();

	// Set all statistical result to zero
	for(i = 0; i< 256; i++)
		histogram[i] = 0;


	for(i = 0; i < image_size; i++){
		offset = 4 * i;
		intensity_ave += data[offset];
		histogram[data[offset]]++;
	}
	
	// Calculate the average intensity, the total black pixels = image_size -
	// white pixels
	intensity_ave /= image_size;
	dark_number = image_size * (1 - intensity_ave / 255.0);

	for(i = 0; i < 256; i++){
		dark_number -= histogram[i];
		if(dark_number <= 0)
			break;
	}

	if(i == 256)
		threshold = 1.0;
	else
		threshold = (double) i / 255.0;

	for(i = 0; i < image_size; i++){
		// threshold the gray data
		if(gray_data[i] < threshold)
			gray_data[i] = 0.0;
		else
			gray_data[i] = 1.0;
		
		// Check the Alpha 
		offset = 4 * i;
		if(data[offset + 3] < 128){
			data[offset + 3] = 0;
			for(j = 0; j < 3; j++)
				data[offset + j] = 0;
		}

		else{
			data[offset + 3] = 255;
			for(j = 0; j < 3; j++)
				data[offset + j] = gray_data[i] * 255.0;
		}
	}

    return true;
}// Dither_Bright



///////////////////////////////////////////////////////////////////////////////
//
//      Dither image using random dithering.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Dither_Random()
{
	// Temporary variable
	int i, j;
	double temp;
	//Starting point
	int offset;

	// Transform the image to gray scale
	To_Grayscale();

	for(i = 0; i < image_size; i++){
		// gray number + [-0.2, 0.2] random number
		temp = gray_data[i] + (0.4 * rand() / RAND_MAX - 0.2);

		// Threshold the result
		if(temp < 0.5)
			gray_data[i] = 0.0;
		else
			gray_data[i] = 1.0;
		
		// Threshold the Alpha value
		offset = 4 * i;
		if(data[offset + 3] < 128){
			data[offset + 3] = 0;
			for(j = 0; j < 3; j++)
				data[offset + j] = 0;
		}

		else{
			data[offset + 3] = 255;
			for(j = 0; j < 3; j++)
				data[offset + j] = gray_data[i] * 255.0;
		}
	}

    return true;
  
}// Dither_Random


///////////////////////////////////////////////////////////////////////////////
//
//      Help the dithering operation in mask method
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::Dither_Mask(double th_mask[][4])
{
	// Temporary variable
	int i, j;
	// The coordinate of point i
	int x, y;
	// The starting point
	int offset;

	for(i = 0; i < image_size; i++){
		x = i % width;
		y = i / width;

		// check the threshold in the mask
		if(gray_data[i] < th_mask[x % 4][y % 4])
			gray_data[i] = 0.0;
		else
			gray_data[i] = 1.0;
		
		// check the Alpha value
		offset = 4 * i;
		if(data[offset + 3] < 128){
			data[offset + 3] = 0;
			for(j = 0; j < 3; j++)
				data[offset + j] = 0;
		}
		else{
			data[offset + 3] = 255;
			for(j = 0; j < 3; j++)
				data[offset + j] = gray_data[i] * 255.0;
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
//
//      Dither the image using ordered dithering.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Dither_Ordered()
{
	double th_mask[][4] = {	{0.1250, 1.0000, 0.1875, 0.8125},
							{0.6250, 0.3750, 0.6875, 0.4375},
							{0.2500, 0.8750, 0.0625, 0.9375},
							{0.7500, 0.5000, 0.5625, 0.3125}};

	// Transform the image to gray scale
	To_Grayscale();

	// Operate in odered dithering with the ordered mask
	Dither_Mask(th_mask);

    return true;
  
}// Dither_Ordered


///////////////////////////////////////////////////////////////////////////////
//
//      Perform clustered dithering of the image.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Dither_Cluster()
{
	double th_mask[][4] = {	{0.7500, 0.3750, 0.6250, 0.2500},
							{0.0625, 1.0000, 0.8750, 0.4375},
							{0.5000, 0.8125, 0.9375, 0.1250},
							{0.1875, 0.5625, 0.3125, 0.6875}};
	// Transform to the gray_scale
	To_Grayscale();

	// Operate in cluster dithering with the cluster mask
	Dither_Mask(th_mask);
    return true;
}// Dither_Cluster


///////////////////////////////////////////////////////////////////////////////
//
//      Perform Floyd-Steinberg dithering on the image.  Return success of 
//  operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Dither_FS()
{
	// Temporary veriable
	int i, j;

	// Current error
	double error;
	// the current offset(x, y) and the offset of next row(x, y+1)
	int    now, next_row;
	// the starting point
	int offset;

	// Transform the image to gray scale
	To_Grayscale();

	// Go through entire image with the zigzag form
	for(i = 0; i < height; i++){

		// For the odd number row 1, 3, 5, 7 and it should 
		// go from end to start
		if(i % 2){
			for(j = width - 1; j >= 0; j--){
				// Calculate the current offset and next row offest
				now = j + i * width;
				next_row = j + (i + 1) * width;

				// Determine whether the current pixel is black or white with
				// the threshold 0.5
				// black the error is current - 0;
				if(gray_data[now] < 0.5){
					error = gray_data[now];
					gray_data[now] = 0;
				}
				// white the error is current - 1.0
				else{
					error = gray_data[now] - 1.0;
					gray_data[now] = 1.0;
				}

				// Relay the error down to the following elements
				// The point is the end of the row
				if(j == 0){
					// The last row means the last element not need to relay
					// Not the last row needs to down, down right
					if(i != height - 1){
						gray_data[next_row]     += (error *  0.3125);
						gray_data[next_row + 1] += (error *  0.1875);
					}
				}// end of the last element of the row
				else if(j == width - 1)	{
					// The last row and needs to relay to the next element
					if(i == height - 1)
						gray_data[now - 1]      += (error *  0.4375);
					// Else need to relay to the next element, down, down_left
					else{
						gray_data[now - 1]      += (error *  0.4375);
						gray_data[next_row]     += (error *  0.3125);
						gray_data[next_row - 1] += (error *  0.0625);
					}
				}// end of the start element of the row
				// In the center
				else{
					// at the last row and need to relay to next element
					if(i == height - 1)
						gray_data[now - 1]      += (error *  0.4375);
					// Needs to relay to next, down, down_left, down_right
					else{
						gray_data[now - 1]      += (error *  0.4375);
						gray_data[next_row + 1] += (error *  0.1875);
						gray_data[next_row]     += (error *  0.3125);
						gray_data[next_row - 1] += (error *  0.0625);
					
					}
				}// end of the if(j...) series
			}//end of the for
		}// end of the odd number row
		// The even row 0, 2, 4 go from left to right
		else{
			for(j = 0; j < width; j++){
				// Calculate the offset of current and next row
				now = j + i * width;
				next_row = j + (i + 1) * width;

				// Determine the current dot
				if(gray_data[now] < 0.5){
					error = gray_data[now];
					gray_data[now] = 0;
				}
				else{
					error = gray_data[now] - 1.0;
					gray_data[now] = 1.0;
				}

				// Relay the error down
				// The start element of the row
				if(j == 0){
					// The last row and needs to relay to next
					if(i == height - 1)
						gray_data[now + 1]      += (error *  0.4375);
					// Else needs to relay to next, down, down_right
					else{
						gray_data[now + 1]      += (error *  0.4375);
						gray_data[next_row]     += (error *  0.3125);
						gray_data[next_row + 1] += (error *  0.0625);
					}
				}// end of the start element
				// The last element
				else if(j == width - 1){
					// The last row means the last element of the image 
					// and no need to relay to next
					// else needs to relay to down, down_left
					if(i != height - 1){
						gray_data[next_row]     += (error *  0.3125);
						gray_data[next_row - 1] += (error *  0.1875);
					}
				}// end of the last element
				else{
					// at the last row needs to relay to next
					if(i == height - 1)
						gray_data[now + 1]      += (error *  0.4375);
					// needs to relay to next, down, down_left, down_right
					else{
						gray_data[now + 1]      += (error *  0.4375);
						gray_data[next_row - 1] += (error *  0.1875);
						gray_data[next_row]     += (error *  0.3125);
						gray_data[next_row + 1] += (error *  0.0625);
					}
				}// end of the if(j..) series
			}// end of for j loop
		}//end of the even number operation
	}//end of for i loop

	for(i = 0; i < image_size; i++){
	// Check the Alpha
 		offset = 4 * i;
		if(data[offset + 3] < 128){
			data[offset + 3] = 0;
			for(j = 0; j < 3; j++)
				data[offset + j] = 0;
		}
		else{
			data[offset + 3] = 255;
			for(j = 0; j < 3; j++)
				data[offset + j] = gray_data[i] * 255.0;
		}
	}

    return true;
}// Dither_FS

///////////////////////////////////////////////////////////////////////////////
//
//      Transform the RGBA to rgb
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::To_rgb(void)
{
	// Temporary variable
	int i, j;
	// The input and output starting point
	int inoffset, outoffset;
	
	rgb_data = new double[image_size * 3];
	for(i = 0; i < image_size; i++){
		inoffset = 4 * i;
		outoffset = 3 * i;
		// if Alpha = 0 everything is zero
		if(!data[inoffset + 3])
			for(j = 0; j < 3; j++)
				rgb_data[outoffset + j] = 0.0;
		// if Alpha != 0 r = R / Alpha
		else
			for(j = 0; j < 3; j++)
				rgb_data[outoffset + j] = (double) data[inoffset + j] / data[inoffset + 3];
	}
	
}

///////////////////////////////////////////////////////////////////////////////
//
//      Perform 5x5 filter mask on rgb
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::filter_mask(double mask[][5])
{
	// temporary array for the rgb
	double *rgb_data_return;
	// temporary variable
	int i, j, k;
	// variable for putting the x, y coordinate and matrix x, y coordinate
	int x, y, pos_x, pos_y;
	// current index
	int now, offset, outoffset;
	double temp_r, temp_g, temp_b;
	
	// Transform data to rgb
	To_rgb();

	rgb_data_return = new double[image_size * 3];
	
	for(i = 0; i < image_size; i++){
		x = i % width;
		y = i / width;

		temp_r = 0;
		temp_g = 0;
		temp_b = 0;
		
		for(j = 0; j < 5; j++){
			pos_x = x + j -2;
			// if the element is outside the edge then mirror back
			// -1 => 1, -2 => 2, w => w - 2, w+1 => w-3
			if(pos_x < 0)
				pos_x = -1 * pos_x;
			else if(pos_x > width - 1)
				pos_x = 2 * (width - 1) - pos_x;
			
			for(k =0; k < 5; k++){
				pos_y = y + k -2;
				// if the element is outside the edge then mirror back
				// -1 => 1, -2 => 2, h => h - 2, h+1 => h-3
				if(pos_y < 0)
					pos_y = -1 * pos_y;
				else if(pos_y > height - 1)
					pos_y = 2* (height - 1) - pos_y;

				// calculate the current position
				now = pos_x + pos_y * width;
				// calculate the array starting point
				offset = now * 3;

				// calculate each multiplication
				temp_r += rgb_data[offset]     * mask[j][k];
				temp_g += rgb_data[offset + 1] * mask[j][k];
				temp_b += rgb_data[offset + 2] * mask[j][k];
			}//end of k
		} // end of j
		
		outoffset = i * 3;
		rgb_data_return[outoffset]     = temp_r; 
		rgb_data_return[outoffset + 1] = temp_g; 
		rgb_data_return[outoffset + 2] = temp_b; 
	}

	delete [] rgb_data;
	rgb_data = rgb_data_return;
}

///////////////////////////////////////////////////////////////////////////////
//
//      Transform filtered rgb back to RGBA
//
///////////////////////////////////////////////////////////////////////////////


void TargaImage::fill_into()
{
	// temporary variable
	int i, j;
	// input and output starting point
	int inoffset, outoffset;

	for(i = 0; i < image_size; i++){
		inoffset  = i * 3;
		outoffset = i * 4;

		// if Alpha = 0 everything is zero
		if(!data[outoffset + 3])
			for(j = 0; j < 3; j++)
				data[outoffset + j] = 0;
			// if Alpha != 0 r = R / Alpha
		else
			for(j = 0; j < 3; j++)
				data[outoffset + j] = rgb_data[inoffset + j] * data[outoffset + 3];
	}

}

///////////////////////////////////////////////////////////////////////////////
//
//      Perform 5x5 box filter on this image.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Filter_Box()
{
	//Box filter maxtrix
	double f_mask[][5] = {	{0.04, 0.04, 0.04, 0.04, 0.04},
							{0.04, 0.04, 0.04, 0.04, 0.04},
							{0.04, 0.04, 0.04, 0.04, 0.04},
							{0.04, 0.04, 0.04, 0.04, 0.04},
							{0.04, 0.04, 0.04, 0.04, 0.04}};
	filter_mask(f_mask);
	fill_into();

    return true;
}// Filter_Box


///////////////////////////////////////////////////////////////////////////////
//
//      Perform 5x5 Bartlett filter on this image.  Return success of 
//  operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Filter_Bartlett()
{
	//Bartlett filter matrix
	double f_mask[][5] = {	{0.0123, 0.0247, 0.0370, 0.0247, 0.0123},
							{0.0247, 0.0494, 0.0741, 0.0494, 0.0247},
							{0.0370, 0.0741, 0.1111, 0.0741, 0.0370},
							{0.0247, 0.0494, 0.0741, 0.0494, 0.0247},
							{0.0123, 0.0247, 0.0370, 0.0247, 0.0123}};

	filter_mask(f_mask);
	fill_into();
    return true;
}// Filter_Bartlett


///////////////////////////////////////////////////////////////////////////////
//
//      Perform 5x5 Gaussian filter on this image.  Return success of 
//  operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Filter_Gaussian()
{
	// Gausian Filter matrix
	double f_mask[][5] = {	{0.0039, 0.0156, 0.0234, 0.0156, 0.0039},
							{0.0156, 0.0625, 0.0937, 0.0625, 0.0156},
							{0.0234, 0.0937, 0.1406, 0.0937, 0.0234},
							{0.0156, 0.0625, 0.0937, 0.0625, 0.0156},
							{0.0039, 0.0156, 0.0234, 0.0156, 0.0039}};

	filter_mask(f_mask);
	fill_into();
    return true;
}// Filter_Gaussian

///////////////////////////////////////////////////////////////////////////////
//
//      Rescale the image into the 0 ~ 1
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::normalize()
{
	// Temporary variable
	int i, j;
	// The starting point
	int offset;

	// The maximum, minimum, and range of the rgb
	double rgb_max[3];
	double rgb_min[3];
	double range[3];

	for(j = 0; j < 3; j++){
		rgb_max[j] = 0.0;
		rgb_min[j] = 0.0;
		range[j] = 0.0;
	}


	for(i = 0; i < image_size; i++){
		offset = 3 * i;
		for(j = 0; j < 3; j++){
			if(rgb_data[offset + j] > rgb_max[j])
				rgb_max[j] = rgb_data[offset + j];
			if(rgb_data[offset + j] < rgb_min[j])
				rgb_min[j] = rgb_data[offset + j];
		}
	}
	for(j = 0; j < 3; j++)
		range[j] = rgb_max[j] - rgb_min[j];

	for(i = 0; i < image_size; i++){
		offset = 3 * i;
		for(j = 0; j < 3; j++)
			rgb_data[offset + j] = (rgb_data[offset + j] - rgb_min[j]) / range[j];	
	}
}


///////////////////////////////////////////////////////////////////////////////
//
//      Perform 5x5 edge detect (high pass) filter on this image.  Return 
//  success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Filter_Edge()
{
	// High pass filter matrix
	double f_mask[][5] = {	{-0.0039, -0.0156, -0.0234, -0.0156, -0.0039},
							{-0.0156, -0.0625, -0.0937, -0.0625, -0.0156},
							{-0.0234, -0.0937,  0.8594, -0.0937, -0.0234},
							{-0.0156, -0.0625, -0.0937, -0.0625, -0.0156},
							{-0.0039, -0.0156, -0.0234, -0.0156, -0.0039}};

	filter_mask(f_mask);
	normalize();
	fill_into();
    return true;
}// Filter_Edge

void TargaImage::truncate()
{
	// Temporary variable
	int i, j;
	// The starting point
	int offset;

	for(i = 0; i < image_size; i++){
		offset = 3 * i;
		for(j = 0; j < 3; j++){
			if(rgb_data[offset + j] > 1.0)
				rgb_data[offset + j] = 1.0;
			if(rgb_data[offset + j] < 0.0)
				rgb_data[offset + j] = 0.0;
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
//
//      Perform a 5x5 enhancement filter to this image.  Return success of 
//  operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Filter_Enhance()
{
	// Edge enhance filter matrix
	double f_mask[][5] = {	{-0.0039, -0.0156, -0.0234, -0.0156, -0.0039},
							{-0.0156, -0.0625, -0.0937, -0.0625, -0.0156},
							{-0.0234, -0.0937,  1.8594, -0.0937, -0.0234},
							{-0.0156, -0.0625, -0.0937, -0.0625, -0.0156},
							{-0.0039, -0.0156, -0.0234, -0.0156, -0.0039}};

	filter_mask(f_mask);
	truncate();
	fill_into();
    return true;
}// Filter_Enhance

///////////////////////////////////////////////////////////////////////////////
//
//      Halve the dimensions of this image.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Half_Size()
{
	// Barklet matrix
	double mask[][5] = {	{0.0123, 0.0247, 0.0370, 0.0247, 0.0123},
							{0.0247, 0.0494, 0.0741, 0.0494, 0.0247},
							{0.0370, 0.0741, 0.1111, 0.0741, 0.0370},
							{0.0247, 0.0494, 0.0741, 0.0494, 0.0247},
							{0.0123, 0.0247, 0.0370, 0.0247, 0.0123}};

	// Temporary array for new data
	double			*rgb_data_return;
	unsigned char	*data_return;
	// the new size, width, height
	int new_image_size;
	int new_w, new_h;
 	// Temporary variable
	int i, j, k, l;
	// x, y coordinate of the desire point and the temporary opperating
	// point of central point
	int x, y, pos_x, pos_y;

	int now, offset, rgb_offset, data_offset;
	double temp_r, temp_g, temp_b;

	// Transform data to rgb
	To_rgb();

	new_w = width / 2;
	new_h = height / 2;
	new_image_size = new_w * new_h;

	rgb_data_return = new double[new_image_size * 3];
	data_return     = new unsigned char[new_image_size * 4];

	for(i = 0; i < new_w; i++){
		for(l = 0; l < new_h; l++){

			// calculate the corresponding x, y coordinate
			x = 2 * i;
			y = 2 * l;

			temp_r = 0;
			temp_g = 0;
			temp_b = 0;

			for(j = 0; j < 5; j++){
				pos_x = x + j -2;
				// if the element is outside the edge then mirror back
				// -1 => 1, -2 => 2, w => w - 2, w+1 => w-3
				if(pos_x < 0)
					pos_x = -1 * pos_x;
				else if(pos_x > width - 1)
					pos_x = 2 * (width - 1) - pos_x;

				for(k =0; k < 5; k++){
					pos_y = y + k -2;
					// if the element is outside the edge then mirror back
					// -1 => 1, -2 => 2, h => h - 2, h+1 => h-3
					if(pos_y < 0)
						pos_y = -1 * pos_y;
					else if(pos_y > height - 1)
						pos_y = 2* (height - 1) - pos_y;

					now = pos_x + pos_y * width;
					offset = now * 3;
					temp_r += rgb_data[offset]     * mask[j][k];
					temp_g += rgb_data[offset + 1] * mask[j][k];
					temp_b += rgb_data[offset + 2] * mask[j][k];
				}// end of k
			} // end of j

			rgb_offset  = (i + l * new_w) * 3;
			data_offset = (i + l * new_w) * 4;

			rgb_data_return[rgb_offset]     = temp_r;
			rgb_data_return[rgb_offset + 1] = temp_g;
			rgb_data_return[rgb_offset + 2] = temp_b;

			// calculate the corresponding alpha value
			offset = (x + y * width) * 4;

			data_return[data_offset]     = temp_r * data[offset + 3];
			data_return[data_offset + 1] = temp_g * data[offset + 3];
			data_return[data_offset + 2] = temp_b * data[offset + 3];
			data_return[data_offset + 3] = data[offset + 3];
			}// end of l
		}// end of i

	width		= new_w;
	height		= new_h;
	image_size	= new_image_size;

	delete [] rgb_data;
	rgb_data = rgb_data_return;

	delete [] data;
	data = data_return;

    return true;
}// Half_Size

///////////////////////////////////////////////////////////////////////////////
//
//      Double the dimensions of this image.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////

bool TargaImage::Double_Size()
{
	// Barklet matrix
	// x is vertical and y is horizontal
	// for x is an int, y is an int
	double maskxy[][5] = {	{0.0123, 0.0247, 0.0370, 0.0247, 0.0123},
							{0.0247, 0.0494, 0.0741, 0.0494, 0.0247},
							{0.0370, 0.0741, 0.1111, 0.0741, 0.0370},
							{0.0247, 0.0494, 0.0741, 0.0494, 0.0247},
							{0.0123, 0.0247, 0.0370, 0.0247, 0.0123}};

	// for x is an a.5 , y is an int
	double masky[][5] = {	{0.0185, 0.0370, 0.0556, 0.0370, 0.0185},
							{0.0370, 0.0740, 0.1111, 0.0740, 0.0370},
							{0.0370, 0.0740, 0.1111, 0.0740, 0.0370},
							{0.0185, 0.0370, 0.0556, 0.0370, 0.0185}};
	// for x is an int, y is an a.5
	double maskx[][4] = {	{0.0185, 0.0370, 0.0370, 0.0185},
							{0.0370, 0.0740, 0.0740, 0.0370},
							{0.0556, 0.1111, 0.1111, 0.0556},
							{0.0370, 0.0740, 0.0740, 0.0370},
							{0.0185, 0.0370, 0.0370, 0.0185}};
	// for x is an a.5, y is an b.5
	double mask[][4] = {	{0.0278, 0.0556, 0.0556, 0.0278},
							{0.0556, 0.1111, 0.1111, 0.0556},
							{0.0556, 0.1111, 0.1111, 0.0556},
							{0.0278, 0.0556, 0.0556, 0.0278}};

	// Temporary variable for image
	double *rgb_data_return;
	unsigned char *data_return;

	// the new size, width, height
	int new_image_size;
	int new_w, new_h;
	// Temporary variable
	int i, j, k, l;
	// x, y coordinate of the desire point and the temporary opperating
	// point of central point
	double x, y;
	int pos_x, pos_y;

	// starting point
	int now, offset, rgb_offset, data_offset;
	// temporary data for rgba
	double temp_r, temp_g, temp_b, temp_a;

	// Transform data to rgb
	To_rgb();

	// calculate the new width and height and image size
	new_w = width * 2;
	new_h = height * 2;
	new_image_size = new_w * new_h;

	rgb_data_return = new double[new_image_size * 3];
	data_return     = new unsigned char[new_image_size * 4];

	for(i = 0; i < new_w; i++){
		for(l = 0; l < new_h; l++){

			x = (double) i / 2.0;
			y = (double) l / 2.0;

			temp_r = 0.0;
			temp_g = 0.0;
			temp_b = 0.0;
			temp_a = 0.0;

			if(i % 2 && l % 2){
				for(j = 0; j < 4; j++){
					pos_x = (double) x + j - 1.5;
					// if the element is outside the edge then mirror back
					// -1 => 1, -2 => 2, w => w - 2, w+1 => w-3
					if(pos_x < 0)
						pos_x = -1 * pos_x;
					else if(pos_x > width - 1)
						pos_x = 2 * (width - 1) - pos_x;

					for(k =0; k < 4; k++){
						pos_y = (double) y + k - 1.5;

						// if the element is outside the edge then mirror back
						// -1 => 1, -2 => 2, h => h - 2, h+1 => h-3
						if(pos_y < 0)
							pos_y = -1 * pos_y;
						else if(pos_y > height - 1)
							pos_y = 2 * (height - 1) - pos_y;

						now = pos_x + pos_y * width;
						offset = now * 3;
						temp_r += rgb_data[offset]     * mask[j][k];
						temp_g += rgb_data[offset + 1] * mask[j][k];
						temp_b += rgb_data[offset + 2] * mask[j][k];
						temp_a += data[now * 4 + 3] * mask[j][k];
					}// end of for k
				} // end of for j
			}// end of if
			else if(i % 2 && !(l % 2)){
				for(j = 0; j < 4; j++){
					pos_x = (double) x + j - 1.5;
					// if the element is outside the edge then mirror back
					// -1 => 1, -2 => 2, w => w - 2, w+1 => w-3
					if(pos_x < 0)
						pos_x = -1 * pos_x;
					else if(pos_x > width - 1)
						pos_x = 2 * (width - 1) - pos_x;

					for(k =0; k < 5; k++){
						pos_y = (double) y + k - 2.0;

						// if the element is outside the edge then mirror back
						// -1 => 1, -2 => 2, h => h - 2, h+1 => h-3
						if(pos_y < 0)
							pos_y = -1 * pos_y;
						else if(pos_y > height - 1)
							pos_y = 2 * (height - 1) - pos_y;

						now = pos_x + pos_y * width;
						offset = now * 3;
						temp_r += rgb_data[offset]     * masky[j][k];
						temp_g += rgb_data[offset + 1] * masky[j][k];
						temp_b += rgb_data[offset + 2] * masky[j][k];
						temp_a += data[now * 4 + 3] * masky[j][k];
					}// end of k
				} // end of for j
			} // end of else if
			else if(!(i % 2)  && l % 2){
				for(j = 0; j < 5; j++){
					pos_x = (double) x + j - 2.0;
					// if the element is outside the edge then mirror back
					// -1 => 1, -2 => 2, w => w - 2, w+1 => w-3
					if(pos_x < 0)
						pos_x = -1 * pos_x;
					else if(pos_x > width - 1)
						pos_x = 2 * (width - 1) - pos_x;

					for(k =0; k < 4; k++){
						pos_y = (double) y + k - 1.5;
	
						// if the element is outside the edge then mirror back
						// -1 => 1, -2 => 2, h => h - 2, h+1 => h-3
						if(pos_y < 0)
							pos_y = -1 * pos_y;
						else if(pos_y > height - 1)
							pos_y = 2 * (height - 1) - pos_y;

						now = pos_x + pos_y * width;
						offset = now * 3;
						temp_r += rgb_data[offset]     * maskx[j][k];
						temp_g += rgb_data[offset + 1] * maskx[j][k];
						temp_b += rgb_data[offset + 2] * maskx[j][k];
						temp_a += data[now * 4 + 3] * maskx[j][k];
					}// end of for k
				} // end of for j
			} // end of else if
			else{
				for(j = 0; j < 5; j++){
					pos_x = (double) x + j - 2.0;
					// if the element is outside the edge then mirror back
					// -1 => 1, -2 => 2, w => w - 2, w+1 => w-3
					if(pos_x < 0)
						pos_x = -1 * pos_x;
					else if(pos_x > width - 1)
						pos_x = 2 * (width - 1) - pos_x;

					for(k =0; k < 5; k++){
						pos_y = (double) y + k - 2.0;

						// if the element is outside the edge then mirror back
						// -1 => 1, -2 => 2, h => h - 2, h+1 => h-3
						if(pos_y < 0)
							pos_y = -1 * pos_y;
						else if(pos_y > height - 1)
							pos_y = 2 * (height - 1) - pos_y;

						now = pos_x + pos_y * width;
						offset = now * 3;
						temp_r += rgb_data[offset]     * maskxy[j][k];
						temp_g += rgb_data[offset + 1] * maskxy[j][k];
						temp_b += rgb_data[offset + 2] * maskxy[j][k];
						temp_a += data[now * 4 + 3] * maskxy[j][k];
					}//end of for k
				} // end of for j
			}// end of else

		rgb_offset = (i + l * new_w) * 3;
		data_offset = (i + l * new_w) * 4;

		rgb_data_return[rgb_offset]     = temp_r;
		rgb_data_return[rgb_offset + 1] = temp_g;
		rgb_data_return[rgb_offset + 2] = temp_b;

		data_return[data_offset + 3] = temp_a + 0.5;
		data_return[data_offset]     = temp_r * data_return[data_offset + 3];
		data_return[data_offset + 1] = temp_g * data_return[data_offset + 3];
		data_return[data_offset + 2] = temp_b * data_return[data_offset + 3];

		}// for l
	}// for i


	width		= new_w;
	height		= new_h;
	image_size	= new_image_size;

	delete [] rgb_data;
	rgb_data = rgb_data_return;

	delete [] data;
	data = data_return;

    return true;
}// Double_Size


///////////////////////////////////////////////////////////////////////////////
//
//      Apply the bartllete to arbitrary point
//
///////////////////////////////////////////////////////////////////////////////
void TargaImage::bartlett_filter(double x, double y, double *rgb_color, unsigned char *color)
{
	// The starting of the matrix and end of the matrix coordinate x, y
	int pos_x_start, pos_x_end;
	int pos_y_start, pos_y_end;

	// The related position x, y in the image
	int pos_x, pos_y;
	// The related Bartlett's coefficient in x direction, y direction, and all
	double coff_x, coff_y, coff, diff_x, diff_y;

	// Temporary data
	double temp_r, temp_g, temp_b, temp_a;
	int k, l;

	// Staring point and image index
	int now, offset;

	if(x > 2.0)
		pos_x_start = ceil(x - 2.0);
	else 
		pos_x_start = x - 2.0;
	pos_x_end   = x + 2.0;

	if(y > 2.0)
		pos_y_start = ceil(y - 2.0);
	else 
		pos_y_start = y - 2.0;

	pos_y_end   = y + 2.0;

	temp_r = 0;
	temp_g = 0;
	temp_b = 0;
	temp_a = 0;

	for(k = pos_x_start; k <= pos_x_end; k++){
		pos_x = k;
		if(pos_x < 0)
			pos_x = -1 * pos_x;
		else if(pos_x > width - 1)
			pos_x = 2 * (width - 1) - pos_x;
		for(l = pos_y_start; l <= pos_y_end; l++){
			pos_y = l;
			if(pos_y < 0)
				pos_y = -1 * pos_y;
			else if(pos_y > height - 1)
				pos_y = 2 * (height - 1) - pos_y;

			now = pos_x + pos_y * width;
			offset = now * 3;

			diff_x = (double) x - pos_x;
			diff_y = (double) y - pos_y;
			
			if(diff_x < 0)
				diff_x = -1.0 * diff_x;

			if(diff_y < 0)
				diff_y = -1.0 * diff_y;

			coff_x = 0.5 * (1.0 - diff_x * 0.5);
			coff_y = 0.5 * (1.0 - diff_y * 0.5);
			coff = coff_x * coff_y;

			temp_r += rgb_data[offset]     * coff;
			temp_g += rgb_data[offset + 1] * coff;
			temp_b += rgb_data[offset + 2] * coff;
			temp_a += data[now * 4 + 3] * coff;

		}// end of l
	}// end of k


	rgb_color[0] = temp_r; 
	rgb_color[1] = temp_g; 
	rgb_color[2] = temp_b; 

	color[3] = 0.5 + temp_a; 
	color[0] = temp_r * color[3]; 
	color[1] = temp_g * color[3]; 
	color[2] = temp_b * color[3]; 

}


///////////////////////////////////////////////////////////////////////////////
//
//      Scale the image dimensions by the given factor.  The given factor is 
//  assumed to be greater than one.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Resize(float scale)
{
	// Temporary holding variable for the image rgb
	double *rgb_data_return;
	unsigned char *data_return;
	// the new size, width, height
	int new_image_size;
	int new_w, new_h;
	// Temporary variable
	int i, j, k;
	// x, y coordinate of the desire point and the temporary opperating 
	// point of central point 
	double x, y;

	// Temporary variable for putting the return value
	double rgb_color[3];
	unsigned char color[4];
	// The starting point in rgb and data array
	int rgb_offset, data_offset;

	// Transform data to rgb
	To_rgb();

	// Calculate the width, height, and size of the new image
	new_w = width * scale;
	new_h = height * scale;
	new_image_size = new_w * new_h;

	rgb_data_return = new double[new_image_size * 3];
	data_return = new unsigned char[new_image_size * 4];

	for(i = 0; i < new_w; i++){
		for(j = 0; j < new_h; j ++){

			x = i / scale;
			y = j / scale;

			bartlett_filter(x, y, rgb_color, color);

			rgb_offset = (i + j * new_w) * 3;
			data_offset = (i + j * new_w) * 4;

			for(k = 0; k < 3; k++)
				rgb_data_return[rgb_offset + k] = rgb_color[k];

			for(k = 0; k < 4; k++)
				data_return[data_offset + k] = color[k];

		}// end of j
	}// end of i

	width		= new_w;
	height		= new_h;
	image_size	= new_image_size;

	delete [] rgb_data;
	rgb_data = rgb_data_return;

	delete [] data;
	data = data_return;

    return true;
}// Resize


///////////////////////////////////////////////////////////////////////////////
//
//      To make the two image the same size
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::zero_pad(int w, int h)
{
  // To put the expansion of the image
  unsigned char *temp;
  
  // Temporary variable
  int i, j;
  // The input starting point and output starting point
  int inoffset, outoffset;
  // x, y coordination
  int x, y;
  // new image_size
  int new_image_size;

  new_image_size = w * h;
  temp = new unsigned char[new_image_size * 4];

  for(i = 0; i < new_image_size * 4; i++)
	  temp[i] = 0;

  // Copy the original image to the padded image
  for(i = 0; i< image_size; i++){
	// Calculate the corresponding x, y coordinate
    x = i % width;
    y = i / width;
	// Calculate the input and output starting point
    inoffset = i * 4;
    outoffset = (x + y * w) * 4;
	// Copy all the image data from old to new
    for(j =0; j < 4; j++)
      temp[outoffset + j] = data[inoffset + j];
  }

	width      = w;
	height     = h;
	image_size = new_image_size;
	delete [] data;
	data = temp;
}


///////////////////////////////////////////////////////////////////////////////
//
//      Composite the current image over the given image.  Return success of 
//  operation.
//
///////////////////////////////////////////////////////////////////////////////

bool TargaImage::Comp_Over(TargaImage* pImage)
{
	// The maximum width, and height in two images
	int max_width, max_height;
	// Temporary variable
	int i, j;
	// The starting point
	int offset;


	if(width < pImage->width)
		max_width = pImage->width;
	else
		max_width = width;

	if(height < pImage->height)
		max_height = pImage->height;
	else
		max_height = height;

	zero_pad(max_width, max_height);
	pImage->zero_pad(max_width, max_height);

    for(i = 0; i < image_size; i++){
		offset = 4 * i;
		// F don't exist but G exist put G in 
		if(!data[offset + 3] && pImage->data[offset + 3]){
			for(j = 0; j < 4; j++)
			data[offset + j] = pImage->data[offset + j];
		}
	}


    return true;
}// Comp_Over


///////////////////////////////////////////////////////////////////////////////
//
//      Composite this image "in" the given image.  See lecture notes for 
//  details.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Comp_In(TargaImage* pImage)
{
	// The maximum width, and height in two images
	int max_width, max_height;
	// Temporary variable
	int i, j;
	// The starting point
	int offset;


	if(width < pImage->width)
		max_width = pImage->width;
	else
		max_width = width;

	if(height < pImage->height)
		max_height = pImage->height;
	else
		max_height = height;

	zero_pad(max_width, max_height);
	pImage->zero_pad(max_width, max_height);


    for(i = 0; i < image_size; i++){
      offset = 4 * i;
	  // if F exist but G does not exist and the let F be off
      if(data[offset + 3] && !pImage->data[offset + 3]){
		for(j = 0; j < 4; j++)
			data[offset + j] = 0;
      }
    }

    return true;
}// Comp_In


///////////////////////////////////////////////////////////////////////////////
//
//      Composite this image "out" the given image.  See lecture notes for 
//  details.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Comp_Out(TargaImage* pImage)
{
	// The maximum width, and height in two images
	int max_width, max_height;
	// Temporary variable
	int i, j;
	// The starting point
	int offset;


	if(width < pImage->width)
		max_width = pImage->width;
	else
		max_width = width;

	if(height < pImage->height)
		max_height = pImage->height;
	else
		max_height = height;

	zero_pad(max_width, max_height);
	pImage->zero_pad(max_width, max_height);

    for(i = 0; i < image_size; i++){
      offset = 4 * i;
	  // if F exist and G also exist and set F be zero
      if(data[offset + 3] && pImage->data[offset + 3]){
		for(j = 0; j < 4; j++)
		  data[offset + j] = 0;
	  }
    }

    return true;
}// Comp_Out


///////////////////////////////////////////////////////////////////////////////
//
//      Composite current image "atop" given image.  Return success of 
//  operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Comp_Atop(TargaImage* pImage)
{
    // The maximum width, and height in two images
	int max_width, max_height;
	// Temporary variable
	int i, j;
	// The starting point
	int offset;


	if(width < pImage->width)
		max_width = pImage->width;
	else
		max_width = width;

	if(height < pImage->height)
		max_height = pImage->height;
	else
		max_height = height;

	zero_pad(max_width, max_height);
	pImage->zero_pad(max_width, max_height);

	for(i = 0; i < image_size; i++){
	  offset = 4 * i;
	  // F exist but G does not, remove F
	  // If F does not exist but G does, add G in
      if(data[offset + 3] && !pImage->data[offset + 3]){
		for(j = 0; j < 4; j++)
			data[offset + j] = 0;
      }
      else if(!data[offset + 3] && pImage->data[offset + 3]){
		for(j = 0; j < 4; j++)
			data[offset + j] = pImage->data[offset + j];
      }
    }

    return true;
}// Comp_Atop


///////////////////////////////////////////////////////////////////////////////
//
//      Composite this image with given image using exclusive or (XOR).  Return
//  success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Comp_Xor(TargaImage* pImage)
{
    	// The maximum width, and height in two images
	int max_width, max_height;
	// Temporary variable
	int i, j;
	// The starting point
	int offset;


	if(width < pImage->width)
		max_width = pImage->width;
	else
		max_width = width;

	if(height < pImage->height)
		max_height = pImage->height;
	else
		max_height = height;

	zero_pad(max_width, max_height);
	pImage->zero_pad(max_width, max_height);

	for(i = 0; i < image_size; i++){
      offset = 4 * i;
	  // if F exist and G exist, set F off
      if(data[offset + 3] && pImage->data[offset + 3]){
		for(j = 0; j < 4; j++)
			data[offset + j] = 0;
      }
	  // if F does not exist and G exist, set G on F
      else if(!data[offset + 3] && pImage->data[offset + 3]){
		for(j = 0; j < 4; j++)
			data[offset + j] = pImage->data[offset + j];
      }
    }
    return true;
}// Comp_Xor

///////////////////////////////////////////////////////////////////////////////
//
//      Calculate the difference bewteen this imag and the given one.  Image 
//  dimensions must be equal.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Difference(TargaImage* pImage)
{
    if (!pImage)
        return false;

    if (width != pImage->width || height != pImage->height)
    {
        cout << "Difference: Images not the same size\n";
        return false;
    }// if

    for (int i = 0 ; i < width * height * 4 ; i += 4)
    {
        unsigned char        rgb1[3];
        unsigned char        rgb2[3];

        RGBA_To_RGB(data + i, rgb1);
        RGBA_To_RGB(pImage->data + i, rgb2);

        data[i] = abs(rgb1[0] - rgb2[0]);
        data[i+1] = abs(rgb1[1] - rgb2[1]);
        data[i+2] = abs(rgb1[2] - rgb2[2]);
        data[i+3] = 255;
    }

    return true;
}// Difference


//////////////////////////////////////////////////////////////////////////////
//
//      Rotate clockwise a angel (x, y) => (x', y')
//
///////////////////////////////////////////////////////////////////////////////

void TargaImage::transform(double angle, double in_x, double in_y, double *out_x, double *out_y)
{
  double cos_a, sin_a;

  cos_a = cos(angle);
  sin_a = sin(angle);

  *out_x = cos_a * in_x + sin_a * in_y;
  *out_y = cos_a * in_y - sin_a * in_x;

}

//////////////////////////////////////////////////////////////////////////////
//
//      Rotate clockwise a angel (x', y') => (x, y)
//
///////////////////////////////////////////////////////////////////////////////


void TargaImage::inv_transform(double angle, double in_x, double in_y, double *out_x, double *out_y)
{
  double cos_a, sin_a;

  cos_a = cos(angle);
  sin_a = sin(angle);

  *out_x = cos_a * in_x - sin_a * in_y;
  *out_y = cos_a * in_y + sin_a * in_x;

}

//////////////////////////////////////////////////////////////////////////////
//
//      Rotate the image clockwise by the given angle.  Do not resize the 
//  image.  Return success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Rotate(float angleDegrees)
{
	// Variable for rotation angle in radian
	double angle;
	// The cos and sin value
	double cos_a, sin_a;

	// The size of new image
	double *rgb_data_return;
	unsigned char *data_return;

	// The center of the picture
	double x_center, y_center;

	// Temporary variable
	int i, j, k;
	// Temporary x, y coordinate
	double x, y, x_prime, y_prime, x_pprime, y_pprime;
	//  Temporary variable for rgb
	double rgb_color[3];
	unsigned char color[4];
	int now, rgb_offset, data_offset;

	int flag = 0;
	
	To_rgb();
	// Transform the angle from degree to radian
	angle = angleDegrees * 3.1415926 / 180.0;

	cos_a = cos(angle);
	sin_a = sin(angle);

  	x_center = (double) width / 2.0;
	y_center = (double) height / 2.0;

 	rgb_data_return = new double[image_size * 3];
	data_return = new unsigned char[image_size * 4];

	for(i = 0; i < width; i++){
		for(j = 0; j < height; j++){
			now = i + j * width;

			// Calculate the starting point of rgb data
			rgb_offset = 3 * now;
			data_offset = 4 * now;

			// Translate from right, up to center
			x_pprime = (double) i - x_center;
			y_pprime = (double) j - y_center;

			// Rotation angle clockwise 
			// from original to current (x', y') => (x'', y'')
			// Inverse transform from (x'', y'') => (x', y');

			x_prime = cos_a * x_pprime + sin_a * y_pprime;
			y_prime = cos_a * y_pprime - sin_a * x_pprime;

			// Translate from center to left, up
			x = x_prime + x_center;
			y = y_prime + y_center;

			// If the point is in the origin picture then get the Bartllete
			// Else give alpha equal to zero
			if(x >= 0 && x <= (width - 1) && y >=0 && y <= height - 1){
				bartlett_filter(x, y, rgb_color, color);
				for(k = 0; k < 3; k++)
					rgb_data_return[rgb_offset + k] = rgb_color[k];
				for(k = 0; k < 4; k++)
					data_return[data_offset + k] = color[k];
			}
			else{
				for(k = 0; k < 3; k++)
					rgb_data_return[rgb_offset + k] = 0.0;
				for(k = 0; k < 4; k++)
					data_return[data_offset + k] = 0;
			}// end of if
		}// end of for j
	}// end of for i
	
	delete [] data;
	delete [] rgb_data;
	data = data_return;
	rgb_data = rgb_data_return;

    return true;
}// Rotate

//////////////////////////////////////////////////////////////////////////////
//
//      Perform a blue screen effect for this image.  Set all blue pixels 
//  ((0, 0, 255) +- 5 for each channel) to have an alpha of zero.  Return 
//  success of operation.
//
///////////////////////////////////////////////////////////////////////////////
bool TargaImage::Blue_Screen()
{
	// Arbitrary close to blue constant
    const unsigned char criteria = 5;
	// The criteria for the blue channel
    unsigned char		blue_c;
	// Temporary variable
    int i, j;
	// image starting point
    int offset;

    blue_c = 255 - criteria;

    for(i = 0; i < image_size; i++){
      offset = 4 * i;
      if(data[offset] < criteria &&
		 data[offset + 1] < criteria &&
		 data[offset + 2] > blue_c){
		  for(j = 0; j < 4; j++)
			data[offset + j] = 0;
	  }// end of if
    }// end of for
    return true;
}// Blue_Screen


//////////////////////////////////////////////////////////////////////////////
//
//      Given a single RGBA pixel return the single RGB equialent composited 
//  with a black background via the second argument. 
//
///////////////////////////////////////////////////////////////////////////////
void TargaImage::RGBA_To_RGB(unsigned char *data, unsigned char *rgb)
{
    const unsigned char	BACKGROUND[3] = { 0, 0, 0 };

    unsigned char  alpha = data[3];

    if (alpha == 0)
    {
        rgb[0] = BACKGROUND[0];
        rgb[1] = BACKGROUND[1];
        rgb[2] = BACKGROUND[2];
    }
    else
    {
	    float	alpha_scale = (float)255 / (float)alpha;
	    int	val;
	    int	i;

	    for (i = 0 ; i < 3 ; i++)
	    {
	        val = (int)floor(data[i] * alpha_scale);
	        if (val < 0)
		    rgb[i] = 0;
	        else if (val > 255)
		    rgb[i] = 255;
	        else
		    rgb[i] = val;
	    }
    }
}// RGA_To_RGB


///////////////////////////////////////////////////////////////////////////////
//
//      Copy this into a new image, reversing the rows as it goes. A pointer
//  to the new image is returned.
//
///////////////////////////////////////////////////////////////////////////////
TargaImage* TargaImage::Reverse_Rows(void)
{
    unsigned char   *dest = new unsigned char[width * height * 4];
    TargaImage	    *result;
    int 	        i, j;

    if (! data)
    	return NULL;

    for (i = 0 ; i < height ; i++)
    {
	    int in_offset = (height - i - 1) * width * 4;
	    int out_offset = i * width * 4;

	    for (j = 0 ; j < width ; j++)
        {
	        dest[out_offset + j * 4] = data[in_offset + j * 4];
	        dest[out_offset + j * 4 + 1] = data[in_offset + j * 4 + 1];
	        dest[out_offset + j * 4 + 2] = data[in_offset + j * 4 + 2];
	        dest[out_offset + j * 4 + 3] = data[in_offset + j * 4 + 3];
        }
    }

    result = new TargaImage(width, height, dest);
    delete[] dest;
    return result;
}// Reverse_Rows


///////////////////////////////////////////////////////////////////////////////
//
//      Clear the image to all black.
//
///////////////////////////////////////////////////////////////////////////////
void TargaImage::ClearToBlack()
{
    memset(data, 0, width * height * 4);
}// ClearToBlack





