///////////////////////////////////////////////////////////////////////////////
//
//      TargaImage.h                            Author:     Stephen Chenney
//                                              Modified:   Eric McDaniel
//                                              Date:       Spring 2002
//
//      Class to manipulate targa images.  You must implement the image 
//  modification functions.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _TARGA_IMAGE_H_
#define _TARGA_IMAGE_H_

#include <Fl/Fl.h>
#include <Fl/Fl_Widget.h>
#include <stdio.h>

#include "Lai_struct.h"

class TargaImage
{
    // members
    public:
		// width of the image in pixels
        int		        width;	    
		// height of the image in pixels
        int		        height;	    
		// pixel data for the image,
		// assumed to be in pre-multiplied RGBA format.
        unsigned char	*data;	  

		// For holding the size of the image
		int             image_size;
		// For holding the data of rgb image
		double			*rgb_data;
		// For holding the data in grayscale
		double			*gray_data;

	// methods
    public:
		//Constructor
	    TargaImage(void);
	    TargaImage(int, int, unsigned char*);

		//Destructor
	    ~TargaImage(void);
		
		// Convert the image to RGB format
        unsigned char*	To_RGB(void);
		
		// save the image to a file
        bool Save_Image(char*);    

		// Load a file and return a pointer to
		// a new TargaImage object.  Returns NULL on failure
        static TargaImage* Load_Image(char*);       

		// Trans form the data to gray scale
        bool To_Grayscale();
		
		// Created the decision tree
		d_tree *median(int n, unsigned char *image_data, int size);
		// Make decision from the decision tree
		void decision_color(int position, d_tree *root, unsigned char *color);
		// release the tree memory space
		void delete_tree(d_tree *root);
        bool Quant_Uniform();
        bool Quant_Populosity();
        bool Quant_Median();

		// Helping function in the dithering operation
		void Dither_Mask(double th_mask[][4]);
        bool Dither_Threshold();
        bool Dither_Random();
        bool Dither_Ordered();
        bool Dither_FS();
        bool Dither_Bright();
        bool Dither_Cluster();

		void zero_pad(int w, int h);
        bool Comp_Over(TargaImage* pImage);
        bool Comp_In(TargaImage* pImage);
        bool Comp_Out(TargaImage* pImage);
        bool Comp_Atop(TargaImage* pImage);
        bool Comp_Xor(TargaImage* pImage);

        bool Difference(TargaImage* pImage);
		
		// Transform the image to rgb
		void To_rgb();
		// Perform the filter mask operation
		void filter_mask(double mask[][5]);
		// Normalize the data back to 0 - 255
		void normalize();
		// put the rgb back to image
		void fill_into();
		// cut the image out of range
		void truncate();

        bool Filter_Box();
        bool Filter_Bartlett();
        bool Filter_Gaussian();
        bool Filter_Edge();
        bool Filter_Enhance();

		// Apply the Bartlett filter to arbitrary point
		void bartlett_filter(double x, double y,double *rgb_color,
								unsigned char *color);
		// rotate clockwise angle in radian
		void transform(double angle, double in_x, double in_y,
						double *out_x, double *out_y);
		// rotate clockwise angle in radian inverse
		void inv_transform(double angle, double in_x, double in_y,
							double *out_x, double *out_y);


        bool Half_Size();
        bool Double_Size();
        bool Resize(float scale);
        bool Blue_Screen();
        bool Rotate(float angleDegrees);

    private:

		// helper function for format conversion
        void RGBA_To_RGB(unsigned char *data, unsigned char *rgb);          
		// reverse the rows of the image, some targas are stored bottom to top
	    TargaImage* Reverse_Rows(void);                                     
		// clear image to all black
        void ClearToBlack();
		
        void Filter_Image(float **filter, int fw, int fh, bool do_alpha);

};


#endif


